////////////////////////////////////////////////////
//Fire PS 1.1 _ VS 2.0 _ Source from AMD / ATI    //
//vvvvable by Digital Slaves_Or3                      //
////////////////////////////////////////////////////

//Transforms

float4x4 tWVP: WORLDVIEWPROJECTION;


//Tweaks

float  Opac <String uiname="Opacity";>;
float  speed;
float  distortion;
float  time;

//Textures

texture flame;
sampler Samp1 = sampler_state
{
   Texture = (flame);
   ADDRESSU = CLAMP;
   ADDRESSV = CLAMP;
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   ADDRESSW = CLAMP;
};
texture noise;
sampler Samp2 = sampler_state
{
   Texture = (noise);
   ADDRESSU = WRAP;
   ADDRESSV = WRAP;
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   ADDRESSW = WRAP;
};
texture opacity;
sampler Samp3 = sampler_state
{
   Texture = (opacity);
   ADDRESSU = CLAMP;
   ADDRESSV = CLAMP;
   MAGFILTER = LINEAR;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   ADDRESSW = CLAMP;
};

//Structure

struct vs2ps
{
   float4 Pos   : POSITION;
   float3 TexC0 : TEXCOORD0;
   float3 TexC1 : TEXCOORD1;
};

// -----------------------------------------------------------------------------
// VERTEXSHADERS
// -----------------------------------------------------------------------------

vs2ps Fire_Vertex_Shader(

          float4 Pos: POSITION,
          float3 TexC0 : TEXCOORD0)
{

   vs2ps Out = (vs2ps)0;

   Out.Pos = mul(Pos, tWVP);

   // Output TexCoord0 directly
   Out.TexC0 = TexC0;

   // Base texture coordinates plus time
   Out.TexC1.x = TexC0.x;
   Out.TexC1.y = TexC0.y + speed * time;

   return Out;
}

// -----------------------------------------------------------------------------
// PIXELSHADERS:
// -----------------------------------------------------------------------------

// Bias and double a value to take it from 0..1 range to -1..1 range

       float4 bx2(float x)
{
   return 2.0f * x - 1.0f;
}

float4 Fire_Pixel_Shader(

       float4 TextC0 : TEXCOORD0,
       float4 TextC1 : TEXCOORD1): COLOR
       
{
   // Sample noise
   float4 noise = tex2D(Samp2, TextC1);

   // Sum of signed noise
   float4 noisefinal = bx2(noise) * distortion ;

   // Perturb base coordinates in direction of noiseSum as function of height (y)
   float4 perturbation = TextC0 + noisefinal * TextC0.y;

   // Sample base and opacity maps with perturbed coordinates
   float4 base = tex2D(Samp1, perturbation);
   
   // Show only Flame
   float4 opacity = tex2D(Samp3, perturbation);

   return base * (opacity * Opac);
}

//--------------------------------------------------------------//
// Technique
//--------------------------------------------------------------//

technique Fire
{
   pass P0
   {
      VertexShader = compile vs_1_1 Fire_Vertex_Shader();
      PixelShader = compile ps_2_0 Fire_Pixel_Shader();
   }

}
